/************************************************************************
New File - pidcontrol.ino - 
created 8Nov 2014 by Eric W. Holzapfel
Too use the PID control library and Pid front end, and pid autotune
9 nov added serial input and pid attempt
************************************************************************/

#include <DualVNH5019MotorShield.h>
#include <PID_v1.h>
#include <PID_AutoTune_v0.h>


DualVNH5019MotorShield md;
//DualVNH5019MotorShield md(10,11,3,5,7,2,4,12);  requires values for 8 pins

//define variables we will be connecting to
double Setpoint, Input, Output;
PID myPID(&Input, &Output, &Setpoint, .4,1,1, DIRECT);

#define InA1            2                 // INA motor pin
#define InB1            4                 // INB motor pin 
#define PWM1            9                 // PWM motor pin	
#define LOOPTIME       100                // PID loop time
#define encodPinA1      11                // encoder A pin, bit 3
#define encodPinB1      12                // encoder B pin, bit 4
#define Vpin            A3  // A2 for pot // battery monitoring analog pin
#define Apin            A0                // motor current monitoring analog pin

// Variables
int speed_req = 350;                      // speed (Set Point)  
int speed_act = 0;                        // speed (actual value)
int PWM_val = 0;                          // (25% = 64; 50% = 127; 75% = 191; 100% = 255)
int voltage = 0;                          // in mV
int current = 0;                          // in mA
volatile long count = 0;                  // rev counter
float Kp =   .4;                          // PID proportional control Gain
float Kd =    1;  

void stopIfFault()
{
  if (md.getM1Fault())
  {
    Serial.println("M1 fault");
    while(1);
  }
//  if (md.getM2Fault())
//  {
//    Serial.println("M2 fault");
//    while(1);
//  }
}

void setup()
{
  Serial.begin(115200);
  Serial.println("Dual VNH5019 Motor Shield");
  analogReference(EXTERNAL);         // Current external ref is 3.3V, appiled to the aREF pin    
  //pinMode(InA1, OUTPUT);
  //pinMode(InB1, OUTPUT);
  //pinMode(PWM1, OUTPUT);
  cli();  // disable interrupts    
    DDRB  |= (0 << 2);  //  Enable pin 10 as INPUT for encodPinA1 set low
    PORTB |= (1 << 2);  //  Set Arduino pin 10 HIGH for pull up (encodPinA1)
    DDRB  |= (0 << 3);  //  Enable pin 2 as INPUT for limit switch - set low
    PORTB |= (1 << 3);  //  Set Arduino pin 11 HIGH for pull up (encodPinB1)
    //pinMode(encodPinA1, INPUT); 
    //pinMode(encodPinB1, INPUT); 
    //digitalWrite(encodPinA1, HIGH);      // turns on pullup resistor
    //digitalWrite(encodPinB1, HIGH);
    PCICR  |= (1 << PCIE0);   // enable group interrupts on PORTB PCINT[7:0]
    PCMSK0 |= (1 << PCINT4);  // enable interrupt pin 10 aka bit 3 was 11,12 now
  sei();  // enable interrupts   using pin 10 for B, and 11 for A

    //setup for PID
    Input = speed_act;     // reads encoder counts
    Setpoint = 400;// speed_req;  // reads the desired speed
    myPID.SetMode(AUTOMATIC);    
    digitalWrite(InA1, LOW);  
    digitalWrite(InB1, HIGH);
    md.init();
    //md.setM1Speed(-speed_req);
}

void getMotorData()     // calculate speed, volts and Amps
{                                         
  static long countAnt = 0;  // last count			  // 64 pulses X 131 gear ratio = 8384, try 32 > 4192
  speed_act = ((count - countAnt)*(60*(1000/LOOPTIME)))/(32*131); // 16 pulses X 29 gear ratio = 464 counts per output shaft rev
  countAnt = count;   
  //voltage = int(analogRead(Vpin) * 3.22 * 12.2/2.2);     // battery voltage: mV=ADC*3300/1024, voltage divider 10K+2K, was 12.2/2.2 for voltage
  //current = int(analogRead(Apin) * 3.22 * .77 *(1000.0/144.0));  // motor current - output: 130mV per Amp  132.0
  current = int(analogRead(Apin)*34);  //md.getM1CurrentMilliamps();
  //5.0 * .77 *(1000.0/144.0));analogRead(Apin) * 34;
  //// 5V / 1024 ADC counts / 144 mV per A = 34 mA per count
  //current = digital_smooth(current, readings);               // remove signal noise
}

void loop()
{
 getParam(); 
 getMotorData(); 
  //for (int i = 0; i <= 400; i++)
  //{
  
  if (speed_req == 350) {
     speed_req = 350;
     md.setM1Speed(speed_req);
     
  } else {     
     Input = speed_act;
     myPID.Compute();
     //speed_req = Output;
     Serial.print("PID Output: ");
     Serial.println(Output);
     //analogWrite(Apin,Output);
      
  }    
       md.setM1Speed(speed_req);
      stopIfFault();
 //   if (i%200 == 100)
 //  {
      //PWM_val = analogRead(Apin);
    
      //Serial.print("M1 current: ");
      //current = int(analogRead(Apin) * 34);
      //Serial.println(md.getM1CurrentMilliamps());
      //Serial.println(current);
      //printMotorInfo();
  //  }
    delay(200);
    printMotorInfo(); 
}

void printMotorInfo()  {               // display data
  
 // if((millis()-lastMilliPrint) >= 500)   {                     
 //   lastMilliPrint = millis();
    Serial.print("SP:");             Serial.println(speed_req);  
    Serial.print("  RPM:");          Serial.println(speed_act);
    //Serial.print("  PWM:");          Serial.println(PWM_val);  
    //Serial.print("  V:");            Serial.print(float(voltage)/1000,1);
    //Serial.print("  mA:");           Serial.println(current);

 //   if (current > CURRENT_LIMIT)            Serial.println("*** CURRENT_LIMIT ***");                
 //   if (voltage > 1000 && voltage*11 < LOW_BAT)   Serial.println("*** LOW_BAT ***");                
 // }
}

int getParam()  {
  char param, cmd;
  if(!Serial.available())    return 0;
  delay(10);                  
  param = Serial.read();                            // get parameter byte
  if(!Serial.available())    return 0;
  cmd = Serial.read();                              // get command byte
  Serial.flush();
  switch (param) {
    case 'v':                                       // adjust speed
      if(cmd=='+')  { 
        speed_req += 12;
        if(speed_req>400) speed_req=400;
        //Input = speed_act;
	//myPID.Compute();
        //speed_req = Output;
        
      }
      if(cmd=='-')    { 
        speed_req -= 12;
        //Input = speed_act;
	//myPID.Compute();
        //speed_req = Output;        
        if(speed_req<0)   speed_req=0;       
      }
      break;
    case 's':                                       // adjust direction
      if(cmd=='+'){
        speed_req = -speed_req;
        digitalWrite(InA1, HIGH);
        digitalWrite(InB1, LOW);
      }
      if(cmd=='-')   {
        speed_req = speed_req;
        digitalWrite(InA1, LOW);
        digitalWrite(InB1, HIGH);
      }
      break;
    case 'o':                                       // user should type "oo"
      digitalWrite(InA1, LOW);
      digitalWrite(InB1, LOW);
      speed_req = 0;
      break;
    default: 
      Serial.println("???");
    }
}


//ISR for Pin 8 was 0 and 1, change to 3 and 2 for pins 11 and 10
ISR (PCINT0_vect)    // handle pin change interrupt for D8-D13;  triggered by 8
{
if(bitRead(PINB,4)== bitRead(PINB,3)) {  //(PINB, x) is the syntax to read the input state from PORTB, bit x 
    //encoderValue++;
    count++;     
  } else {
    //encoderValue--;
    count--;     
  } 
}
